/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include "kiss_fft.h"
#include "df.h"

df::df( int set_method, int set_winlen, double *set_window, double set_thresh ) {

	/* Declarations */
	int n;

	/* Set parameters */
	method = set_method;
	winlen = set_winlen;
	winlen2 = winlen >> 1;
	thresh = set_thresh;

	/* Init window */
	window = new double[winlen];
	for (n=0;n<winlen;n++) window[n] = set_window[n];

	/* Init fft */
	infft = new kiss_fft_cpx[winlen];
	outfft = new kiss_fft_cpx[winlen];
	cfg = kiss_fft_alloc( winlen , 0, 0, 0 );

	/* Init arrays */
	amp = new double[winlen2];
	amp_old = new double[winlen2];
	phase = new double[winlen2];
	phase_old = new double[winlen2];
	phase_oldold = new double[winlen2];
	for (n=0;n<winlen2;n++) {
		amp[n] = 0.0;
		amp_old[n] = 0.0;
		phase[n] = 0.0;
		phase_old[n] = 0.0;
		phase_oldold[n] = 0.0;
	}
}

df::~df( void ) {

	/* Clean memory */
	free( cfg );
	delete[]( infft );
	delete[]( outfft );
	delete[]( amp );
	delete[]( amp_old );
	delete[]( phase );
	delete[]( phase_old );
	delete[]( phase_oldold );
	delete[]( window );
}

double df::compute( double *in ) {

	/* Declarations */
	int n;
	double dfval = 0;
	double temp1,temp2,temp3;

	/* Window + Swap left and right half */
	for (n=0;n<winlen2;n++) {
		infft[n].r = in[n+winlen2] * window[n+winlen2];
		infft[n].i = 0.0;
		infft[n+winlen2].r = in[n] * window[n];
		infft[n+winlen2].i = 0.0;
	}

	/* FFT */
	kiss_fft( cfg , infft , outfft );

	/* Amplitude and phase */
	for (n=0;n<winlen2;n++) {
		amp[n] = sqrt( outfft[n+winlen2].r*outfft[n+winlen2].r + outfft[n+winlen2].i*outfft[n+winlen2].i );
		phase[n] = atan2( outfft[n+winlen2].i, outfft[n+winlen2].r );
	}

	/* Detection function */
	dfval = 0.0;
	switch (method) {
		case 1:
			for (n=0;n<winlen2;n++) {
				if (amp[n]>thresh) {
					temp1 = phase[n] - 2*phase_old[n] + phase_oldold[n];
					temp1 = ((temp1+M_PI)-floor((temp1+M_PI)/(-2*M_PI))*(-2*M_PI))+M_PI;
					temp2 = amp_old[n] - amp[n]*cos(temp1);
					temp3 = - amp[n]*sin(temp1);
					dfval += sqrt( temp2*temp2 + temp3*temp3 );
				}
			}
			break;
		case 2:
			for (n=0;n<winlen2;n++) {
				if (amp[n]>thresh) {
					temp1 = amp[n]*amp[n]-amp_old[n]*amp_old[n];
					if (temp1<0) temp1 = -temp1;
					dfval += sqrt( temp1 );
				}
			}
			break;
		case 3:
			for (n=0;n<winlen2;n++) {
				if (amp[n]>thresh) {
					temp1 = phase[n] - 2*phase_old[n] + phase_oldold[n];
					temp2 = ((temp1+M_PI)-floor((temp1+M_PI)/(-2*M_PI))*(-2*M_PI))+M_PI;
					if (temp2<0) temp2 = -temp2;
					dfval += temp2;
				}
			}
			break;
	}

	/* Update arrays */
	for (n=0;n<winlen2;n++) {
		amp_old[n] = amp[n];
		phase_oldold[n] = phase_old[n];
		phase_old[n] = phase[n];
	}

	/* Return */
	return dfval;
}

