/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>
#include "mex.h"
#include "df.h"
#include "resample.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	/* Declarations */
	double *signal,*dfout,*output,*win;
	double thresh,sum;
	int numsamples,numchannels;
	int method,winlen,hopsize;
	int n,numframes,upratio;
	df *detfun;
	mxArray *p;
	mxArray *temp;
	bool setwin = false;
	double eps = pow(2,-52);

	/* Check number of arguments */
	if (nrhs<1) mexErrMsgTxt( "Not enough input arguments.");
	if (nlhs>1) mexErrMsgTxt( "Too many output arguments.");

	/* Input Signal */
	signal = mxGetPr(prhs[0]);
	numsamples = (int)mxGetDimensions(prhs[0])[0];
	numchannels = (int)mxGetDimensions(prhs[0])[1];
	if (numchannels!=1) mexErrMsgTxt( "Input signal must be one column vector.");

	/* Parameters */
	if (nrhs>=2) {
		if (!mxIsStruct(prhs[1])) mexErrMsgTxt("Parameters must be a structure.");
		p = (mxArray*)prhs[1];
	} else {
		p = mxCreateStructMatrix(0,0,0,NULL);
	}
	method = 1;
	temp = mxGetField(p, 0, "method");
	if (temp!=NULL) method = (int)mxGetPr(temp)[0];
	winlen = 2048;
	temp = mxGetField(p, 0, "winlen");
	if (temp!=NULL) winlen = (int)mxGetPr(temp)[0];
	hopsize = 1024;
	temp = mxGetField(p, 0, "hopsize");
	if (temp!=NULL) hopsize = (int)mxGetPr(temp)[0];
	win = (double*) mxCalloc( winlen, sizeof(double) );
	for (n=0;n<winlen;n++) win[n] = 0.5*(1-cos(2*M_PI*n/winlen));
	temp = mxGetField(p, 0, "win");
	if (temp!=NULL) {
		mxFree( win );
		win = mxGetPr(temp);
		setwin = true;
	}
	if (method==1)
		thresh = -1;
	else
		thresh = 0.1;
	temp = mxGetField(p, 0, "thresh");
	if (temp!=NULL) thresh = mxGetPr(temp)[0];
	upratio = 2;
	temp = mxGetField(p, 0, "upratio");
	if (temp!=NULL) upratio = (int)mxGetPr(temp)[0];

	/* Init detection function object */
	detfun = new df(method,winlen,win,thresh);

	/* Init output */
	numframes = (int)ceil((double)(numsamples-winlen)/(double)hopsize);
	dfout = (double*)mxCalloc(numframes,sizeof(double));
	if (nlhs>0) {
		plhs[0] = mxCreateDoubleMatrix(1,numframes*upratio,mxREAL);
		output = mxGetPr( plhs[0] );
	}

	/* Computation loop */
	for (n=0;n<numframes;n++)
		dfout[n] = detfun->compute( signal+hopsize*n );

	if (nlhs>0) {
		/* Upsample */
		upsample( dfout, numframes, mxGetPr(plhs[0]), upratio );

		/* Force postive values */
		sum = 0;
		for (n=0;n<numframes*upratio;n++) {
			if (mxGetPr(plhs[0])[n]<0) mxGetPr(plhs[0])[n] = 0;
			sum += mxGetPr(plhs[0])[n]+eps;
		}

		/* Normalize */
		for (n=0;n<numframes*upratio;n++)
			mxGetPr(plhs[0])[n] = (mxGetPr(plhs[0])[n]+eps)/sum;
	}

	/* Clean */
	mxFree( dfout );
	if (setwin==false) mxFree( win );
	delete( detfun );
}

